//******************************************************************************************
//* Spring (Helicoid/Catenoid) Minimal Surface, by Paul Nylander, bugman123.com, 6/4/05
//******************************************************************************************

#declare Alpha=0.8175188542971654;
#declare sina=sin(Alpha); #declare cosa=cos(Alpha); #declare dz=2*pi*cosa;

camera{location <0,15,0> look_at <0,0,0> right x*image_width/image_height up z sky z}
light_source{<0,15,0>,1}

//Basic Functions
#declare colors=array[7] {<0,1,0,0>,<1/6,1,1,0>,<1/3,0,1,0>,<1/2,0,1,1>,<2/3,0,0,1>,<5/6,1,0,1>,<1,1,0,0>};
#macro hue(h) #local n=dimension_size(colors,1); #local i=0;
 #while(i<n & colors[min(i,n-1)].x<h) #local i=i+1; #end
 #if(i>0 & i<n) #local x1=colors[i-1].x; #local x2=colors[i].x; #local c=((x2-h)*colors[i-1]+(h-x1)*colors[i])/(x2-x1); #else #local c=colors[min(i,n-1)]; #end
 <c.y,c.z,c.t>
#end

//Parametric Plotting, function f must be defined before this function can be called
#macro ParametricPlot3D(u1,u2,du, v1,v2,dv) #local imax=int((u2-u1)/du); #local jmax=int((v2-v1)/dv);
 #local Mesh=array[imax+1][jmax+1]; #local i=0;
 #while(i<=imax) #local U=u1+i*du; #local j=0; #while(j<=jmax) #local Mesh[i][j]=f(U,v1+j*dv); #local j=j+1; #end #local i=i+1; #end
 DrawMesh(Mesh)
#end
#macro quad(p1,n1, p2,n2, p3,n3, p4,n4)
 #if(vlength(p3-p1)<vlength(p4-p2))
  smooth_triangle{p1,n1,p2,n2,p3,n3} smooth_triangle{p1,n1,p3,n3,p4,n4}
 #else
  smooth_triangle{p1,n1,p2,n2,p4,n4} smooth_triangle{p2,n2,p3,n3,p4,n4}
 #end
#end
#macro DrawMesh(Mesh)
 #local imax=dimension_size(Mesh,1); #local jmax=dimension_size(Mesh,2); #local normals=array[imax][jmax];
 #local i=0;
 #while(i<imax) #local j=0; #while(j<jmax) #local normals[i][j]=<0,0,0>; #local j=j+1; #end #local i=i+1; #end
 #local i=0;
 #while(i<imax-1) #local j=0; // adapted from Tim Wenclawiak's Make_Normals function
  #while(j<jmax-1)
   #local p0=(Mesh[i][j]+Mesh[i][j+1]+Mesh[i+1][j]+Mesh[i+1][j+1])/4;
   #local N=vcross(Mesh[i+1][j]-p0,Mesh[i+1][j+1]-p0);
   #local E=vcross(Mesh[i+1][j+1]-p0,Mesh[i][j+1]-p0);
   #local S=vcross(Mesh[i][j+1]-p0,Mesh[i][j]-p0);
   #local W=vcross(Mesh[i][j]-p0,Mesh[i+1][j]-p0);
   #local normals[i][j]=normals[i][j]+S+W;
   #local normals[i+1][j]=normals[i+1][j]+N+W;
   #local normals[i][j+1]=normals[i][j+1]+S+E;
   #local normals[i+1][j+1]=normals[i+1][j+1]+N+E;
   #local j=j+1;
  #end
  #local i=i+1;
 #end
 #local i=0;
 #while(i<imax) #local j=0; #while(j<jmax) #local normals[i][j]=vnormalize(normals[i][j]); #local j=j+1; #end #local i=i+1; #end
 #local i=0;
 #while (i<imax-1) #local j=0;
  #while (j<jmax-1)
   quad(
    Mesh[i  ][j  ],normals[i  ][j  ], Mesh[i+1][j  ],normals[i+1][j  ],
    Mesh[i+1][j+1],normals[i+1][j+1], Mesh[i  ][j+1],normals[i  ][j+1]
   )
   #local j=j+1;
  #end
  #local i=i+1;
 #end
#end

//Helicoid/Catenoid Surface
#declare vmax=2.25;
#macro f(U,V)
 #local sinu=sin(U); #local cosu=cos(U); #local sinhv=sinh(V); #local coshv=cosh(V);
 <cosa*sinhv*sinu+sina*coshv*cosu, -cosa*sinhv*cosu+sina*coshv*sinu, U*cosa+V*sina>
#end
union{
 ParametricPlot3D(-2*pi,3*pi,pi/9, -vmax,vmax,vmax/3)
 pigment{radial
  color_map{[0 rgbt <1,0,0,0.5>][1/6 rgbt <1,1,0,0.5>][1/3 rgbt <0,1,0,0.5>][0.5 rgbt <0,1,1,0.5>][2/3 rgbt <0,0,1,0.5>][5/6 rgbt <1,0,1,0.5>][1 rgbt <1,0,0,0.5>]}
  rotate <90,0,0>
 }
 finish{reflection 0.4}
}

//Tube
#declare rtube=0.25;
#macro TubePlot3D(u1,u2,du) #local U=u1; #while(U<u2-du/2) #local p=g(U); sphere{p,rtube} cylinder{p,g(U+du),rtube} #local U=U+du; #end sphere{g(U),rtube} #end
#macro g(U) f(U,-vmax) #end
union{TubePlot3D(-2*pi,4*pi,pi/9) pigment{rgb 1} finish{reflection 0.5}}
